!include "MUI2.nsh"
!include "nsProcess.nsh"
!include "TextReplace.nsh"
!include "Library.nsh"
!include "LogicLib.nsh"
!include "WinVer.nsh"
!include "x64.nsh"
!include "FileFunc.nsh"

!define PRODUCT_NAME "MDA"
!define PRODUCT_VERSION "0.9.51"

!define MUI_LANGDLL_REGISTRY_ROOT HKLM
!define MUI_LANGDLL_REGISTRY_KEY "Software\Millistream\MDA"
!define MUI_LANGDLL_REGISTRY_VALUENAME "NSIS:Language"

Var StartMenuFolder
Var InstalledDir
Var HaveMYSQL
Var HavePQ
Var StoppedService

;--------------------------------
;General
  Name "${PRODUCT_NAME}"
  OutFile "mda-${PRODUCT_VERSION}.exe"
  InstallDirRegKey HKLM "Software\Millistream\MDA" "Install Directory"
  LicenseForceSelection off
  RequestExecutionLevel admin
  SetCompressor /SOLID lzma
  ShowInstDetails show

;--------------------------------
;Interface Configuration
  !define MUI_HEADERIMAGE
  !define MUI_HEADERIMAGE_BITMAP "files\banner\banner.bmp"
  !define MUI_ABORTWARNING

;--------------------------------
;Pages
  !define MUI_LICENSEPAGE_TEXT_TOP " "
  !define MUI_LICENSEPAGE_TEXT_BOTTOM " "
  !define MUI_LICENSEPAGE_BUTTON $(lbutton)
  !insertmacro MUI_PAGE_LICENSE $(license)
  !insertmacro MUI_PAGE_COMPONENTS
  !insertmacro MUI_PAGE_DIRECTORY

  ;Start Menu Folder Page Configuration
  !define MUI_STARTMENUPAGE_REGISTRY_ROOT "HKLM" 
  !define MUI_STARTMENUPAGE_REGISTRY_KEY "Software\Millistream\MDA" 
  !define MUI_STARTMENUPAGE_REGISTRY_VALUENAME "Start Menu Folder"
  !define MUI_STARTMENUPAGE_DEFAULTFOLDER "Millistream"
  
  !insertmacro MUI_PAGE_STARTMENU Application $StartMenuFolder

  !insertmacro MUI_PAGE_INSTFILES
  
  !insertmacro MUI_UNPAGE_CONFIRM
  !insertmacro MUI_UNPAGE_INSTFILES
  
;--------------------------------
;Languages
 
  !insertmacro MUI_LANGUAGE "English"
  !insertmacro MUI_LANGUAGE "Swedish"

  LangString lbutton ${LANG_ENGLISH} "Continue"
  LangString lbutton ${LANG_SWEDISH} "Fortstt"
  LicenseLangString license ${LANG_ENGLISH} "files\license\gpl-3.0.txt"
  LicenseLangString license ${LANG_SWEDISH} "files\license\gpl-3.0.sv.txt"

  LangString serviceDesc ${LANG_ENGLISH} "Updates a SQL database with market data from Millistream"
  LangString serviceDesc ${LANG_SWEDISH} "Uppdaterar en SQL-databas med marknadsdata frn Millistream"

  LangString settings ${LANG_ENGLISH} "Edit mda.config"
  LangString settings ${LANG_SWEDISH} "Redigera mda.config"

  LangString console ${LANG_ENGLISH} "Run MDA in Console Mode"
  LangString console ${LANG_SWEDISH} "Kr MDA i konsollge"

  LangString checkmdarunning ${LANG_ENGLISH} "Looking for a running MDA."
  LangString checkmdarunning ${LANG_SWEDISH} "Kollar om MDA krs."

  LangString mdaisrunning ${LANG_ENGLISH} "Found a running MDA. Trying to stop the MDA system service."
  LangString mdaisrunning ${LANG_SWEDISH} "MDA verkar kras. Frsker stoppa MDA-systemtjnsten."

  LangString mdaisstillrunning ${LANG_ENGLISH} "MDA is still running, waiting for it to stop."
  LangString mdaisstillrunning ${LANG_SWEDISH} "MDA kr fortfarande, vntar p att den skall stoppas."

  LangString mdanostop ${LANG_ENGLISH} "Could not stop the MDA system service:"
  LangString mdanostop ${LANG_SWEDISH} "Kunde inte stoppa MDA-systemtjnsten:"

  LangString mdanotstopped ${LANG_ENGLISH} "The MDA appears to be running. Please stop it and retry the installation."
  LangString mdanotstopped ${LANG_SWEDISH} "MDA verkar kras. Vnligen stng den och starta installationsprogrammet igen."

;--------------------------------
;Installer Sections

Section "MDA Core Files" SecCore
SectionIn RO
  ${If} ${RunningX64}
    ${DisableX64FSRedirection}
  ${EndIf}

  SetShellVarContext all
  SetOutPath "$INSTDIR\MDA"

  DetailPrint "$(checkmdarunning)"
  ${nsProcess::FindProcess} "mda.exe" $R0
  ${If} $R0 == 0
    DetailPrint "$(mdaisrunning)"
    SimpleSC::StopService "Millistream MDA" 1 240
    Pop $0
    ${If} $0 != 0
      Push $0
      SimpleSC::GetErrorMessage
      Pop $0
      DetailPrint "$(mdanostop) $0"
    ${Else}
      StrCpy $StoppedService 1
    ${EndIf}

    ${nsProcess::FindProcess} "mda.exe" $R0
    ${If} $R0 == 0
      DetailPrint "$(mdaisstillrunning)"
try_again:
      Sleep 3000
      ${nsProcess::FindProcess} "mda.exe" $R0
      ${If} $R0 == 0
        Goto try_again
      ${EndIf}
    ${EndIf}
  ${EndIf}

  File "..\..\mod_perfmon\mod_perfmon.ini"
  File "..\..\mod_perfmon\CounterOffsets.h"
  File "/oname=MySQL.sql" "..\..\mod_mysql\script.sql"
  File "/oname=MySQL (v5.6.4+).sql" "..\..\mod_mysql\script2.sql"
  File "/oname=PostgreSQL.sql" "..\..\mod_postgresql\script.sql"
  File "/oname=MSSQL Server.sql" "..\..\mod_mssql\script.sql"

  ${If} ${RunningX64}
    File "files\bin64\mda.exe"
  ${Else}
    File "files\bin32\mda.exe"
  ${EndIf}

  IfFileExists "$INSTDIR\MDA\mda.config" config_exists 0
    File "files\mda.config"
    ${textreplace::ReplaceInFile} "$INSTDIR\MDA\mda.config" "$INSTDIR\MDA\mda.config" "[INSTALLPATH]" "$INSTDIR" "/S=1 /C=0 /AO=1" $0
    AccessControl::DisableFileInheritance "$INSTDIR\MDA\mda.config"
    AccessControl::RevokeOnFile /NOINHERIT "$INSTDIR\MDA\mda.config" "(S-1-5-32-545)" "FullAccess"

config_exists:
  IfFileExists "$INSTDIR\MDA\tabledef.txt" tabledef_exists 0
    File "/oname=tabledef.txt" "..\..\etc\mda.tabledef"

tabledef_exists:
  SetOutPath "$INSTDIR\MDA\Modules"

  ${If} ${RunningX64}
    File "files\bin64\mod_blackhole.dll"
    File "files\bin64\mod_mssql.dll"
    File "files\bin64\mod_mysql.dll"
    File "files\bin64\mod_perfmon.dll"
    File "files\bin64\mod_postgresql.dll"
    File "files\bin64\mod_odbc.dll"
  ${Else}
    File "files\bin32\mod_blackhole.dll"
    File "files\bin32\mod_mssql.dll"
    File "files\bin32\mod_mysql.dll"
    File "files\bin32\mod_perfmon.dll"
    File "files\bin32\mod_postgresql.dll"
    File "files\bin32\mod_odbc.dll"
  ${EndIf}

  ; Install the Shared DLLs
  ReadRegStr $0 HKLM "Software\Millistream\MDA" "core_dlls"

  ${If} ${RunningX64}
    ${DisableX64FSRedirection}
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared64\libcrypto-1_1-x64.dll" "$SYSDIR\libcrypto-1_1-x64.dll" $SYSDIR
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared64\libexpat-1.dll" "$SYSDIR\libexpat-1.dll" $SYSDIR
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared64\libhashit.dll" "$SYSDIR\libhashit.dll" $SYSDIR
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared64\libmdf-0.dll" "$SYSDIR\libmdf-0.dll" $SYSDIR
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared64\zlib1.dll" "$SYSDIR\zlib1.dll" $SYSDIR
    ${EnableX64FSRedirection}
  ${Else}
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared32\libcrypto-1_1.dll" "$SYSDIR\libcrypto-1_1.dll" $SYSDIR
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared32\libexpat-1.dll" "$SYSDIR\libexpat-1.dll" $SYSDIR
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared32\libgcc_s_dw2-1.dll" "$SYSDIR\libgcc_s_dw2-1.dll" $SYSDIR
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared32\libhashit.dll" "$SYSDIR\libhashit.dll" $SYSDIR
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared32\libmdf-0.dll" "$SYSDIR\libmdf-0.dll" $SYSDIR
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared32\zlib1.dll" "$SYSDIR\zlib1.dll" $SYSDIR
  ${EndIf}

  WriteRegStr HKLM "Software\Millistream\MDA" "core_dlls" "1"

  ;Store installation folder
  WriteRegStr HKLM "Software\Millistream\MDA" "Install Directory" $INSTDIR

  ;Create uninstaller
  WriteUninstaller "$INSTDIR\MDA\Uninstall.exe"

  ; Write the uninstall keys for Windows
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\Millistream MDA" "DisplayName" "Millistream Database Application"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\Millistream MDA" "UninstallString" '"$INSTDIR\MDA\Uninstall.exe"'
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\Millistream MDA" "NoModify" 1
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\Millistream MDA" "NoRepair" 1

  ; Write Event Logger keys for Windows
  WriteRegStr HKLM "SYSTEM\CurrentControlSet\Services\Eventlog\Application\Millistream Database Application" "EventMessageFile" "$INSTDIR\MDA\mda.exe"
  WriteRegDWORD HKLM "SYSTEM\CurrentControlSet\Services\Eventlog\Application\Millistream Database Application" "TypesSupported" 7

  ; Write Performance Monitor Keys
  ExecWait 'unlodctr "Millistream Database Application"'
  WriteRegStr HKLM "SYSTEM\CurrentControlSet\Services\Millistream Database Application\Performance" "Library" "$INSTDIR\MDA\modules\mod_perfmon.dll"
  WriteRegStr HKLM "SYSTEM\CurrentControlSet\Services\Millistream Database Application\Performance" "Open" "OpenPerformanceData"
  WriteRegStr HKLM "SYSTEM\CurrentControlSet\Services\Millistream Database Application\Performance" "Collect" "CollectPerformanceData"
  WriteRegStr HKLM "SYSTEM\CurrentControlSet\Services\Millistream Database Application\Performance" "Close" "ClosePerformanceData"
  DeleteRegValue HKLM "SYSTEM\CurrentControlSet\Services\Millistream Database Application\Performance" "Disable Performance Counters"

  SetOutPath "$INSTDIR\MDA"
  ExecWait 'lodctr mod_perfmon.ini'

  SimpleSC::InstallService "Millistream MDA" "Millistream Database Application" "16" "3" '$INSTDIR\MDA\mda.exe --config "$INSTDIR\MDA\mda.config"' "" "" ""
  SimpleSC::SetServiceDescription "Millistream MDA" "$(serviceDesc)"
  SimpleSC::SetServiceFailure "Millistream MDA" 0 "" "" 1 60000 1 60000 1 60000

  !insertmacro MUI_STARTMENU_WRITE_BEGIN Application

  ;Create shortcuts
  CreateDirectory "$SMPROGRAMS\$StartMenuFolder\MDA"
  CreateShortCut "$SMPROGRAMS\$StartMenuFolder\MDA\$(settings).lnk" "Notepad.exe" "$INSTDIR\MDA\mda.config"
  CreateShortCut "$SMPROGRAMS\$StartMenuFolder\MDA\$(console).lnk" "$INSTDIR\MDA\mda.exe" '--config "$INSTDIR\MDA\mda.config"'

  ${If} ${AtLeastWinVista}
    ShellLink::SetRunAsAdministrator "$SMPROGRAMS\$StartMenuFolder\MDA\$(settings).lnk"
    Pop $0
    ShellLink::SetRunAsAdministrator "$SMPROGRAMS\$StartMenuFolder\MDA\$(console).lnk"
    Pop $0
  ${Endif}

  !insertmacro MUI_STARTMENU_WRITE_END

  # Remove files needed to install the performance monitor counters
  Delete "$INSTDIR\MDA\mod_perfmon.ini"
  Delete "$INSTDIR\MDA\CounterOffsets.h"
SectionEnd

Section /o "MySQL Libraries" SecMySQL
  ReadRegStr $0 HKLM "Software\Millistream\MDA" "mysql_dlls"

  ${If} ${RunningX64}
    ${DisableX64FSRedirection}
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared64\mariadb.dll" "$SYSDIR\mariadb.dll" $SYSDIR
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared64\libwinpthread-1.dll" "$SYSDIR\libwinpthread-1.dll" $SYSDIR
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared64\libssl-1_1-x64.dll" "$SYSDIR\libssl-1_1-x64.dll" $SYSDIR
    ${EnableX64FSRedirection}
  ${Else}
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared32\mariadb.dll" "$SYSDIR\mariadb.dll" $SYSDIR
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared32\libwinpthread-1.dll" "$SYSDIR\libwinpthread-1.dll" $SYSDIR
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared32\libssl-1_1.dll" "$SYSDIR\libssl-1_1.dll" $SYSDIR
  ${EndIf}

  WriteRegStr HKLM "Software\Millistream\MDA" "mysql_dlls" "1"
SectionEnd

Section /o "PostgreSQL Libraries" SecPostgreSQL
  ReadRegStr $0 HKLM "Software\Millistream\MDA" "postgresql_dlls"

  ${If} ${RunningX64}
    ${DisableX64FSRedirection}
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared64\libpq.dll" "$SYSDIR\libpq.dll" $SYSDIR
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared64\libssl-1_1-x64.dll" "$SYSDIR\libssl-1_1-x64.dll" $SYSDIR
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared64\libintl-8.dll" "$SYSDIR\libintl-8.dll" $SYSDIR
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared64\libiconv-2.dll" "$SYSDIR\libiconv-2.dll" $SYSDIR
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared64\libxml2-2.dll" "$SYSDIR\libxml2-2.dll" $SYSDIR
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared64\liblzma-5.dll" "$SYSDIR\liblzma-5.dll" $SYSDIR
    ${EnableX64FSRedirection}
  ${Else}
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared32\libpq.dll" "$SYSDIR\libpq.dll" $SYSDIR
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared32\libssl-1_1.dll" "$SYSDIR\libssl-1_1.dll" $SYSDIR
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared32\libintl-8.dll" "$SYSDIR\libintl-8.dll" $SYSDIR
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared32\libiconv-2.dll" "$SYSDIR\libiconv-2.dll" $SYSDIR
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared32\libxml2-2.dll" "$SYSDIR\libxml2-2.dll" $SYSDIR
    !insertmacro InstallLib DLL $0 NOREBOOT_PROTECTED "files\shared32\liblzma-5.dll" "$SYSDIR\liblzma-5.dll" $SYSDIR
  ${EndIf}

  WriteRegStr HKLM "Software\Millistream\MDA" "postgresql_dlls" "1"
SectionEnd

Function .onInit
  ${If} ${RunningX64}
    StrCpy $INSTDIR "$PROGRAMFILES64\Millistream"
  ${Else}
    StrCpy $INSTDIR "$PROGRAMFILES\Millistream"
  ${EndIf}

  ReadRegStr $InstalledDir HKLM "Software\Millistream\MDA" "Install Directory"
  Iferrors +2 0
  StrCpy $INSTDIR $InstalledDir

  !insertmacro MUI_LANGDLL_DISPLAY

  ReadRegStr $0 HKLM "Software\Millistream\MDA" "mysql_dlls"
  Iferrors +2 0
  SectionSetFlags ${SecMySQL} 1

  ReadRegStr $0 HKLM "Software\Millistream\MDA" "postgresql_dlls"
  Iferrors +2 0
  SectionSetFlags ${SecPostgreSQL} 1

  ; See if we where called with /mysql or /postgres to enable those sections in say a silent installer
  ${GetParameters} $R1
  ${GetOptions} $R1 "/mysql" $R2
  ${IfNot} ${Errors}
    SectionSetFlags ${SecMySQL} 1
  ${EndIf}
  ${GetOptions} $R1 "/postgres" $R2
  ${IfNot} ${Errors}
    SectionSetFlags ${SecPostgreSQL} 1
  ${EndIf}

FunctionEnd

Function .onInstSuccess
  ${If} $StoppedService == 1
    DetailPrint "Starting the MDA system service again since we stopped it during install..."
    SimpleSC::StartService "Millistream MDA" "" 30
    Pop $0
    ${If} $0 != 0
      Push $0
      SimpleSC::GetErrorMessage
      Pop $0
      DetailPrint "$0"
    ${EndIf}
  ${EndIf}
FunctionEnd

;--------------------------------
;Descriptions

  ;Language strings
  LangString DESC_SecCore ${LANG_ENGLISH} "Core files needed by the Millistream Database Application"
  LangString DESC_SecCore ${LANG_SWEDISH} "Grundfiler som behvs fr Millistream Database Application"

  LangString DESC_SecMySQL ${LANG_ENGLISH} "MySQL/MariaDB support libraries. Needed if you are going to connect to MySQL/MariaDB database"
  LangString DESC_SecMySQL ${LANG_SWEDISH} "Stdfiler fr MySQL/MariaDB. Behvs fr att koppla upp sig mot en MySQL/MariaDB databas"

  LangString DESC_SecPostgreSQL ${LANG_ENGLISH} "PostgreSQL support libraries. Only needed if you are going to connect to a remote PostgreSQL server and PostgreSQL hasn't been installed to this computer."
  LangString DESC_SecPostgreSQL ${LANG_SWEDISH} "Stdfiler fr PostgreSQL. Behvs enbart fr att koppla upp sig mot en PostgreSQL databas som inte r installerad p denna dator"

  ;Assign language strings to sections
  !insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
    !insertmacro MUI_DESCRIPTION_TEXT ${SecCore} $(DESC_SecCore)
    !insertmacro MUI_DESCRIPTION_TEXT ${SecMySQL} $(DESC_SecMySQL)
    !insertmacro MUI_DESCRIPTION_TEXT ${SecPostgreSQL} $(DESC_SecPostgreSQL)
  !insertmacro MUI_FUNCTION_DESCRIPTION_END
 
;--------------------------------
;Uninstaller Section

Function un.onInit

  !insertmacro MUI_UNGETLANGUAGE

FunctionEnd

Section "Uninstall"

  SetShellVarContext all
  ReadRegStr $InstalledDir HKLM "Software\Millistream\MDA" "Install Directory"
  ReadRegStr $HaveMYSQL HKLM "Software\Millistream\MDA" "mysql_dlls"
  ReadRegStr $HavePQ HKLM "Software\Millistream\MDA" "postgresql_dlls"

  SimpleSC::StopService "Millistream MDA" 1 60

wait_for_mda_shutdown:
  ${nsProcess::FindProcess} "mda.exe" $R0
  StrCmp $R0 "0" 0 +3
    Sleep 200
    Goto wait_for_mda_shutdown

  SimpleSC::RemoveService "Millistream MDA"

  RMDir /r "$InstalledDir\MDA"
  RMDir "$InstalledDir"

  ; Remove Windows uninstall keys
  DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\Millistream MDA"

  ; Remove Event Log keys
  DeleteRegKey HKLM "SYSTEM\CurrentControlSet\Services\Eventlog\Application\Millistream Database Application"

  ; Remove Performance Monitor Keys
  ExecWait 'unlodctr "Millistream Database Application"'
  DeleteRegKey HKLM "SYSTEM\CurrentControlSet\Services\Millistream Database Application"

  !insertmacro MUI_STARTMENU_GETFOLDER Application $StartMenuFolder

  RMDir /r "$SMPROGRAMS\$StartMenuFolder\MDA"
  RMDir "$SMPROGRAMS\$StartMenuFolder"

  DeleteRegKey HKLM "SOFTWARE\Millistream\MDA"
  DeleteRegKey /ifempty HKLM "SOFTWARE\Millistream"

  ${If} ${RunningX64}
    ${DisableX64FSRedirection}
  ${EndIf}

  ${If} ${RunningX64}
    !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\libcrypto-1_1-x64.dll
    !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\libexpat-1.dll
    !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\libhashit.dll
    !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\libmdf-0.dll
    !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\zlib1.dll
  ${Else}
    !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\libcrypto-1_1.dll
    !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\libexpat-1.dll
    !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\libgcc_s_dw2-1.dll
    !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\libhashit.dll
    !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\libmdf-0.dll
    !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\zlib1.dll
  ${EndIf}

  ; Mysql Libraries
  ${If} $HaveMYSQL == "1"
    ${If} ${RunningX64}
      !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\mariadb.dll
      !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\libwinpthread-1.dll
      !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\libssl-1_1-x64.dll
    ${Else}
      !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\mariadb.dll
      !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\libwinpthread-1.dll
      !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\libssl-1_1.dll
    ${EndIf}
  ${EndIf}

  ; PostgreSQL Libraries
  ${If} $HavePQ == "1"
    ${If} ${RunningX64}
      !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\libpq.dll
      !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\libssl-1_1-x64.dll
      !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\libintl-8.dll
      !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\libiconv-2.dll
      !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\libxml2-2.dll
      !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\liblzma-5.dll
    ${Else}
      !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\libpq.dll
      !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\libssl-1_1.dll
      !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\libintl-8.dll
      !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\libiconv-2.dll
      !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\libxml2-2.dll
      !insertmacro UnInstallLib DLL SHARED NOREBOOT_PROTECTED $SYSDIR\liblzma-5.dll
    ${EndIf}
  ${EndIf}

SectionEnd
