CREATE DATABASE [millistream]
GO
USE [millistream]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO

CREATE TABLE [dbo].[trades] (
	[insref] [numeric](20, 0) NOT NULL,
	[tradeprice] [float] NULL,
	[tradeyield] [float] NULL,
	[tradequantity] [float] NULL,
	[tradereference] [varchar](255) COLLATE Latin1_General_CS_AS NOT NULL,
	[tradecode] [int] NOT NULL,
	[mmt] [char](14) COLLATE Latin1_General_CS_AS NULL,
	[tradetype] [varchar](10) NULL,
	[buyer] [varchar](64) NULL,
	[seller] [varchar](64) NULL,
	[executedside] [char](1) NULL,
	[canceltime] [time](6) NULL,
	[agreementtime] [time](6) NULL,
	[agreementdate] [date] NULL,
	[executionvenue] [varchar](255) NULL,
	[currency] [varchar](4) NULL,
	[time] [time](6) NOT NULL,
	[date] [date] NOT NULL,
	CONSTRAINT [trades_pk] PRIMARY KEY CLUSTERED ([insref] ASC, [date] ASC, [tradereference] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[stats] (
	[name] [varchar](255) NOT NULL,
	[value] [varchar](255) NOT NULL
	CONSTRAINT [stats_pk] PRIMARY KEY CLUSTERED ([name] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[quotes] (
	[insref] [numeric](20, 0) NOT NULL,
	[bidprice] [float] NULL,
	[bidquantity] [float] NULL,
	[askprice] [float] NULL,
	[askquantity] [float] NULL,
	[lastprice] [float] NULL,
	[dayhighprice] [float] NULL,
	[daylowprice] [float] NULL,
	[quantity] [numeric](20, 0) NULL,
	[internalquantity] [numeric](20, 0) NULL,
	[offbookquantity] [numeric](20, 0) NULL,
	[darkquantity] [numeric](20, 0) NULL,
	[turnover] [numeric](38, 7) NULL,
	[internalturnover] [numeric](38, 7) NULL,
	[offbookturnover] [numeric](38, 7) NULL,
	[darkturnover] [numeric](38, 7) NULL,
	[numtrades] [numeric](20, 0) NULL,
	[openprice] [float] NULL,
	[bidyield] [float] NULL,
	[askyield] [float] NULL,
	[lastyield] [float] NULL,
	[openyield] [float] NULL,
	[dayhighyield] [float] NULL,
	[daylowyield] [float] NULL,
	[nav] [float] NULL,
	[tis] [float] NULL,
	[vwap] [float] NULL,
	[pluspaid] [int] NULL,
	[minuspaid] [int] NULL,
	[unchangedpaid] [int] NULL,
	[duration] [float] NULL,
	[settlementprice] [float] NULL,
	[time] [time](0) NULL,
	[date] [date] NULL,
	CONSTRAINT [quotes_pk] PRIMARY KEY CLUSTERED ([insref] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[quotesex] (
	[insref] [numeric](20, 0) NOT NULL,
	[bidprice] [float] NULL,
	[bidquantity] [float] NULL,
	[askprice] [float] NULL,
	[askquantity] [float] NULL,
	[lastprice] [float] NULL,
	[dayhighprice] [float] NULL,
	[daylowprice] [float] NULL,
	[quantity] [numeric](20, 0) NULL,
	[turnover] [numeric](38, 7) NULL,
	[numtrades] [numeric](20, 0) NULL,
	[time] [time](0) NULL,
	[date] [date] NULL,
	CONSTRAINT [quotesex_pk] PRIMARY KEY CLUSTERED ([insref] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[pricehistory] (
	[insref] [numeric](20, 0) NOT NULL,
	[type] [smallint] NULL,
	[closebidprice] [float] NULL,
	[closeaskprice] [float] NULL,
	[closetradeprice] [float] NULL,
	[closeprice] [float] NULL,
	[closedayhighprice] [float] NULL,
	[closedaylowprice] [float] NULL,
	[closequantity] [numeric](20, 0) NULL,
	[closeinternalquantity] [numeric](20, 0) NULL,
	[closeoffbookquantity] [numeric](20, 0) NULL,
	[closedarkquantity] [numeric](20, 0) NULL,
	[closeturnover] [numeric](38, 7) NULL,
	[closeinternalturnover] [numeric](38, 7) NULL,
	[closeoffbookturnover] [numeric](38, 7) NULL,
	[closedarkturnover] [numeric](38, 7) NULL,
	[openprice] [float] NULL,
	[numtrades] [numeric](20, 0) NULL,
	[openyield] [float] NULL,
	[closebidyield] [float] NULL,
	[closeaskyield] [float] NULL,
	[closeyield] [float] NULL,
	[closedayhighyield] [float] NULL,
	[closedaylowyield] [float] NULL,
	[closenav] [float] NULL,
	[closetis] [float] NULL,
	[closevwap] [float] NULL,
	[mcap] [numeric](38, 7) NULL,
	[settlementprice] [float] NULL,
	[time] [time](0) NOT NULL,
	[date] [date] NOT NULL,
	CONSTRAINT [pricehistory_pk] PRIMARY KEY CLUSTERED ([insref] ASC, [date] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[adjustedpricehistory] (
	[insref] [numeric](20, 0) NOT NULL,
	[type] [smallint] NULL,
	[closebidprice] [float] NULL,
	[closeaskprice] [float] NULL,
	[closetradeprice] [float] NULL,
	[closeprice] [float] NULL,
	[closedayhighprice] [float] NULL,
	[closedaylowprice] [float] NULL,
	[closequantity] [numeric](20, 0) NULL,
	[closeinternalquantity] [numeric](20, 0) NULL,
	[closeoffbookquantity] [numeric](20, 0) NULL,
	[closedarkquantity] [numeric](20, 0) NULL,
	[closeturnover] [numeric](38, 7) NULL,
	[closeinternalturnover] [numeric](38, 7) NULL,
	[closeoffbookturnover] [numeric](38, 7) NULL,
	[closedarkturnover] [numeric](38, 7) NULL,
	[openprice] [float] NULL,
	[numtrades] [numeric](20, 0) NULL,
	[openyield] [float] NULL,
	[closebidyield] [float] NULL,
	[closeaskyield] [float] NULL,
	[closeyield] [float] NULL,
	[closedayhighyield] [float] NULL,
	[closedaylowyield] [float] NULL,
	[closenav] [float] NULL,
	[closetis] [float] NULL,
	[closevwap] [float] NULL,
	[mcap] [numeric](38, 7) NULL,
	[settlementprice] [float] NULL,
	[time] [time](0) NOT NULL,
	[date] [date] NOT NULL,
	CONSTRAINT [adjustedpricehistory_pk] PRIMARY KEY CLUSTERED ([insref] ASC, [date] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[orderbooks] (
	[insref] [numeric](20, 0) NOT NULL,
	[price] [float] NOT NULL,
	[quantity] [float] NULL,
	[side] [char](1) NOT NULL,
	[numorders] [smallint] NULL,
	[counterparties] [varchar](255) NULL,
	[mmo] [smallint] NULL,
	[time] [time](6) NOT NULL,
	[date] [date] NOT NULL,
	CONSTRAINT [orderbook_pk] PRIMARY KEY CLUSTERED ([insref] ASC, [price] ASC, [side] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[mbo] (
	[insref] [numeric](20, 0) NOT NULL,
	[id] [numeric](20, 0) NOT NULL,
	[id_source] [varchar](255) NULL,
	[side] [smallint] NULL,
	[priority] [numeric](20, 0) NULL,
	[price] [float] NULL,
	[quantity] [float] NULL,
	[participant] [varchar](255) NULL,
	[mmo] [smallint] NULL,
	[time] [time](6) NOT NULL,
	[date] [date] NOT NULL,
	CONSTRAINT [mbo_pk] PRIMARY KEY CLUSTERED ([insref] ASC, [id] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[news] (
	[newsref] [numeric](20, 0) NOT NULL,
	[headline] [nvarchar](255) NOT NULL,
	[text] [nvarchar](max) NULL,
	[newstype] [smallint] NULL,
	[subject] [smallint] NULL,
	[regulatory] [smallint] NULL,
	[language] [char](2) NULL,
	[newsid] [char](36) NOT NULL,
	[replaces] [char](36) NULL,
	[originalcode] [varchar](255) NULL,
	[rank] [smallint] NULL,
	[impact] [smallint] NULL,
	[package] [numeric](20, 0) NULL,
	[time] [time](0) NOT NULL,
	[date] [date] NOT NULL,
	CONSTRAINT [news_pk] PRIMARY KEY CLUSTERED ([newsid] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]
GO
CREATE INDEX [IX_NEWS] ON [news] ([newsref],[date]);
GO

CREATE TABLE [dbo].[listmappings] (
	[list] [numeric](20, 0) NOT NULL,
	[insref] [numeric](20, 0) NOT NULL
	CONSTRAINT [listmappings_pk] PRIMARY KEY CLUSTERED ([list] ASC, [insref] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO
CREATE INDEX [IX_LISTMAPPINGS] ON [listmappings] ([insref]);
GO

CREATE TABLE [dbo].[instruments] (
	[insref] [numeric](20, 0) NOT NULL,
	[name] [nvarchar](255) NULL,
	[symbol] [varchar](50) NULL,
	[isin] [char](12) NULL,
	[boardlot] [int] NULL,
	[instrumenttype] [smallint] NULL,
	[instrumentsubtype] [smallint] NULL,
	[instrumentclass] [smallint] NULL,
	[instrumentsubclass] [varchar](30) NULL,
	[instrumentsubsubclass] [varchar](120) NULL,
	[derivativeindicator] [smallint] NULL,
	[exercisetype] [smallint] NULL,
	[issuecurrency] [varchar](4) NULL,
	[tradecurrency] [varchar](4) NULL,
	[basecurrency] [varchar](4) NULL,
	[quotecurrency] [varchar](4) NULL,
	[issuedate] [date] NULL,
	[strikedate] [date] NULL,
	[strikeprice] [float] NULL,
	[marketplace] [numeric](20, 0) NULL,
	[primarymarketplace] [numeric](20, 0) NULL,
	[submarket] [numeric](20, 0) NULL,
	[sector] [numeric](20,0) NULL,
	[company] [numeric](20, 0) NULL,
	[fundppmcode] [int] NULL,
	[fundpmicode] [varchar](11) NULL,
	[fundcompany] [numeric](20, 0) NULL,
	[country] [char](2) NULL,
	[freetextcomment] [varchar](255) NULL,
	[numberofshares] [numeric](20, 0) NULL,
	[numberofprefshares] [numeric](20, 0) NULL,
	[nominalvalue] [float] NULL,
	[mic] [char](5) NULL,
	[operatingmic] [char](5) NULL,
	[tradestate] [smallint] NULL,
	[chairman] [nvarchar](60) NULL,
	[ceo] [nvarchar](255) NULL,
	[website] [nvarchar](255) NULL,
	[orgnum] [varchar](11) NULL,
	[orgnum_no] [numeric](9, 0) NULL,
	[cvr] [int] NULL,
	[y_tunnus] [varchar](9) NULL,
	[kennitala] [numeric](10, 0) NULL,
	[cik] [numeric](10, 0) NULL,
	[shortdescription] [nvarchar](512) NULL,
	[description] [nvarchar](max) NULL,
	[genderceo] [char](1) NULL,
	[genderchairman] [char](1) NULL,
	[birthyearceo] [smallint] NULL,
	[birthyearchairman] [smallint] NULL,
	[address] [nvarchar](255) NULL,
	[postalcode] [nvarchar](255) NULL,
	[city] [nvarchar](50) NULL,
	[telephone] [varchar](40) NULL,
	[fax] [varchar](40) NULL,
	[email] [nvarchar](255) NULL,
	[importantevents] [nvarchar](max) NULL,
	[contractsize] [float] NULL,
	[baseratio] [float] NULL,
	[sourceid] [varchar](32) NULL,
	[issuer] [varchar](255) NULL,
	[issuername] [nvarchar](255) NULL,
	[ongoingcharge] [float] NULL,
	[fundyearlymgmtfee] [float] NULL,
	[fundppmfee] [float] NULL,
	[fundleverage] [float] NULL,
	[funddirection] [varchar](5) NULL,
	[fundbenchmark] [nvarchar](255) NULL,
	[fundbenchmarkinsref] [numeric](20, 0) NULL,
	[fundppmtype] [varchar](255) NULL,
	[fundrisk] [smallint] NULL,
	[prospectus] [nvarchar](max) NULL,
	[geofocusregion] [nvarchar](20) NULL,
	[geofocuscountry] [nvarchar](20) NULL,
	[openinterest] [float] NULL,
	[geniumid] [int] NULL,
	[cusip] [varchar](9) NULL,
	[wkn] [varchar](6) NULL,
	[ucits] [smallint] NULL,
	[inceptiondate] [date] NULL,
	[barrierprice] [float] NULL,
	[ceoadmissiondate] [date] NULL,
	[chairmanadmissiondate] [date] NULL,
	[coupondate] [date] NULL,
	[couponrate] [float] NULL,
	[couponfrequency] [smallint] NULL,
	[fundmcap] [float] NULL,
	[fundminadditionalamount] [float] NULL,
	[fundminstartamount] [float] NULL,
	[fundminsubscriptionamount] [float] NULL,
	[fundperformancefee] [float] NULL,
	[purchasefee] [float] NULL,
	[salesfee] [float] NULL,
	[totalfee] [float] NULL,
	[morningstarrating] [smallint] NULL,
	[tradedthroughdate] [date] NULL,
	[dividendtype] [smallint] NULL,
	[dividendfrequency] [smallint] NULL,
	[pricingfrequency] [smallint] NULL,
	[eusipa] [int] NULL,
	[convertfromdate] [date] NULL,
	[converttodate] [date] NULL,
	[conversionprice] [float] NULL,
	[specialcondition] [int] NULL,
	[settlementtype] [smallint] NULL,
	[votingpower] [float] NULL,
	[cap] [float] NULL,
	[pricetype] [smallint] NULL,
	[volumedimension] [smallint] NULL,
	[quotingtype] [smallint] NULL,
	[asiantailstart] [date] NULL,
	[asiantailend] [date] NULL,
	[logotype] [nvarchar](max) NULL,
	[ticktable] [numeric](20, 0) NULL,
	[contractvalue] [float] NULL,
	[financinglevel] [float] NULL,
	[unitlinkcompany] [varchar](255) NULL,
	[participationrate] [float] NULL,
	[issueprice] [float] NULL,
	[fiinstitutenumber] [numeric](20, 0) NULL,
	[kiid] [nvarchar](max) NULL,
	[cfi] [char](6) NULL,
	[maxlevel] [float] NULL,
	[outstandingamount] [float] NULL,
	[interestrate] [float] NULL,
	[marketmaker] [varchar](255) NULL,
	[marketopendays] [int] NULL,
	[marketopen] [time](0) NULL,
	[marketclose] [time](0) NULL,
	[marketearlyclose] [time](0) NULL,
	[legalstructure] [varchar](255) NULL,
	[productcode] [varchar](20) NULL,
	[latestyearendreport] [varchar](4) NULL,
	[csr] [float] NULL,
	[activeshare] [float] NULL,
	[trackingerror] [float] NULL,
	[figi] [varchar](12) NULL,
	[figicomposite] [varchar](12) NULL,
	[figishareclass] [varchar](12) NULL,
	[figisecuritytype] [varchar](40) NULL,
	[daycountconvention] [smallint] NULL,
	[expirationtype] [smallint] NULL,
	[tenor] [varchar](5) NULL,
	[swingfactor] [float] NULL,
	[swingmethod] [smallint] NULL,
	[time] [time](0) NULL,
	[date] [date] NULL,
	CONSTRAINT [instruments_pkey] PRIMARY KEY CLUSTERED ([insref] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[corporateactions] (
	[insref] [numeric](20, 0) NOT NULL,
	[name] [nvarchar] (255) NULL,
	[symbol] [varchar] (255) NULL,
	[isin] [char] (12) NULL,
	[marketplace] [numeric] (20,0) NULL,
	[list] [varchar] (255) NULL,
	[underlyingid] [varchar] (255) NULL,
	[type] [smallint] NOT NULL,
	[subtype] [smallint] NULL,
	[dividend] [float] NULL,
	[netdividend] [float] NULL,
	[adjustmentfactor] [float] NULL,
	[redemptionprice] [float] NULL,
	[numberofshares] [numeric](20, 0) NULL,
	[numberofprefshares] [numeric](20, 0) NULL,
	[numberofsharesdelta] [numeric](20, 0) NULL,
	[newshares] [float] NULL,
	[oldshares] [float] NULL,
	[subscriptionprice] [float] NULL,
	[period] [varchar](19) NULL,
	[fiscalperiod] [varchar](19) NULL,
	[nominalvalue] [float] NULL,
	[boardlot] [int] NULL,
	[recorddate] [date] NULL,
	[paymentdate] [date] NULL,
	[announcementdate] [date] NULL,
	[tid] [float] NULL,
	[closeprice] [float] NULL,
	[issuecurrency] [varchar](4) NULL,
	[tradecurrency] [varchar](4) NULL,
	[freetextcomment1] [nvarchar](255) NULL,
	[freetextcomment2] [nvarchar](255) NULL,
	[freetextcomment3] [nvarchar](255) NULL,
	[freetextcomment4] [nvarchar](255) NULL,
	[freetextcomment5] [nvarchar](255) NULL,
	[votingpowerprc] [float] NULL,
	[capitalprc] [float] NULL,
	[mcap] [float] NULL,
	[eventlink] [char](36) NULL,
	[eventlinklanguages] [varchar](255) NULL,
	[source] [char](15) NOT NULL,
	[date] [date] NOT NULL,
	[time] [time](0) NOT NULL,
	CONSTRAINT [corporateactions_pkey] PRIMARY KEY CLUSTERED ([insref] ASC, [date] ASC, [time] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO
CREATE INDEX [IX_CORPORATEACTIONS_1] ON [corporateactions] ([insref],[type],[date]);
CREATE INDEX [IX_CORPORATEACTIONS_2] ON [corporateactions] ([insref],[type],[subtype],[date]);
GO

CREATE TABLE [dbo].[fundamentals] (
	[insref] [numeric](20, 0) NOT NULL,
	[sales] [float] NULL,
	[ebit] [float] NULL,
	[ebita] [float] NULL,
	[ebitda] [float] NULL,
	[financialincome] [float] NULL,
	[financialcost] [float] NULL,
	[ptp] [float] NULL,
	[np] [float] NULL,
	[eps] [float] NULL,
	[dilutedeps] [float] NULL,
	[netfinancialincome] [float] NULL,
	[equityratio] [float] NULL,
	[numberofshares] [float] NULL,
	[operatingcashflow] [float] NULL,
	[gp] [float] NULL,
  	[intangibleasset] [float] NULL,
  	[goodwill] [float] NULL,
  	[fixedasset] [float] NULL,
  	[financialasset] [float] NULL,
  	[noncurrentasset] [float] NULL,
  	[inventory] [float] NULL,
  	[othercurrentasset] [float] NULL,
  	[accountsreceivable] [float] NULL,
  	[otherreceivables] [float] NULL,
  	[shortterminv] [float] NULL,
  	[cce] [float] NULL,
  	[currentassets] [float] NULL,
  	[totalassets] [float] NULL,
  	[shequity] [float] NULL,
  	[minorityinterest] [float] NULL,
  	[provisions] [float] NULL,
  	[ltliabilities] [float] NULL,
  	[curliabilities] [float] NULL,
  	[totsheqliabilities] [float] NULL,
  	[nibl] [float] NULL,
  	[ibl] [float] NULL,
  	[cashflowbwc] [float] NULL,
  	[cashflowawc] [float] NULL,
  	[cashflowia] [float] NULL,
  	[cashflowfa] [float] NULL,
  	[cashflowtotal] [float] NULL,
  	[interestincome] [float] NULL,
  	[otherfinancialincome] [float] NULL,
  	[interestexpense] [float] NULL,
  	[otherfinancialexpense] [float] NULL,
  	[minorityinterestres] [float] NULL,
  	[accountspayable] [float] NULL,
	[eventlink] [char](36) NULL,
	[eventlinklanguages] [varchar](255) NULL,
  	[numemployees] [float] NULL,
  	[dividend] [float] NULL,
  	[incomepropman] [float] NULL,
	[period] [varchar](7) NOT NULL,
	[fiscalperiod] [char](19) NULL,
	[currency] [char](3) NOT NULL,
	[adjustmentfactor] [float] NULL,
	[source] [char](15) NOT NULL,
	[date] [date] NOT NULL,
	[announcementdate] [date] NULL,
	CONSTRAINT [fundamentals_pkey] PRIMARY KEY CLUSTERED ([insref] ASC, [period] ASC, [currency] ASC, [source] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO
CREATE INDEX [IDX_FUNDAMENTALS] ON [fundamentals] ([insref],[period]);
GO

CREATE TABLE [dbo].[estimates] (
	[insref] [numeric](20, 0) NOT NULL,
	[period] [varchar](7) NOT NULL,
	[field] [varchar](30) NOT NULL,
	[aspect] [varchar](10) NOT NULL,
	[type] [varchar](20) NOT NULL,
	[unit] [varchar](255),
	[average] [float] NULL,
	[min] [float] NULL,
	[max] [float] NULL,
	[count] [int] NULL,
	[currency] [char](3) NOT NULL,
	[source] [char](15) NOT NULL,
	[date] [date] NOT NULL,
	[time] [time](0) NOT NULL,
	CONSTRAINT [estimates_pkey] PRIMARY KEY CLUSTERED ([insref] ASC, [period] ASC, [field] ASC, [aspect] ASC, [type] ASC, [currency] ASC, [source] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[estimateshistory] (
	[insref] [numeric](20, 0) NOT NULL,
	[period] [varchar](7) NOT NULL,
	[field] [varchar](30) NOT NULL,
	[aspect] [varchar](10) NOT NULL,
	[type] [varchar](20) NOT NULL,
	[unit] [varchar](255),
	[average] [float] NULL,
	[min] [float] NULL,
	[max] [float] NULL,
	[count] [int] NULL,
	[currency] [char](3) NOT NULL,
	[source] [char](15) NOT NULL,
	[date] [date] NOT NULL,
	[time] [time](0) NOT NULL,
	CONSTRAINT [estimateshistory_pkey] PRIMARY KEY CLUSTERED ([insref] ASC, [date] ASC, [period] ASC, [field] ASC, [aspect] ASC, [type] ASC, [currency] ASC, [source] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[performance] (
	[insref] [numeric](20, 0) NOT NULL,
	[closebidprice1d] [float] NULL,
	[closebidprice1w] [float] NULL,
	[closeprice] [float] NULL,
	[closeprice1d] [float] NULL,
	[closeprice1w] [float] NULL,
	[closeprice2w] [float] NULL,
	[closeprice1m] [float] NULL,
	[closeprice3m] [float] NULL,
	[closeprice6m] [float] NULL,
	[closeprice9m] [float] NULL,
	[closeprice1y] [float] NULL,
	[closeprice2y] [float] NULL,
	[closeprice3y] [float] NULL,
	[closeprice5y] [float] NULL,
	[closeprice10y] [float] NULL,
	[closepricewtd] [float] NULL,
	[closepricemtd] [float] NULL,
	[closepriceqtd] [float] NULL,
	[closepriceytd] [float] NULL,
	[closepricepytd] [float] NULL,
	[closepriceld] [float] NULL,
	[closebidyield1d] [float] NULL,
	[closebidyield1w] [float] NULL,
	[closeyield1d] [float] NULL,
	[closeyield1w] [float] NULL,
	[closeyield2w] [float] NULL,
	[closeyield1m] [float] NULL,
	[closeyield3m] [float] NULL,
	[closeyield6m] [float] NULL,
	[closeyield9m] [float] NULL,
	[closeyield1y] [float] NULL,
	[closeyield2y] [float] NULL,
	[closeyield3y] [float] NULL,
	[closeyield5y] [float] NULL,
	[closeyield10y] [float] NULL,
	[closeyieldwtd] [float] NULL,
	[closeyieldmtd] [float] NULL,
	[closeyieldqtd] [float] NULL,
	[closeyieldytd] [float] NULL,
	[closeyieldpytd] [float] NULL,
	[closeyieldld] [float] NULL,
	[ath] [float] NULL,
	[atl] [float] NULL,
	[highprice1y] [float] NULL,
	[lowprice1y] [float] NULL,
	[highprice3y] [float] NULL,
	[lowprice3y] [float] NULL,
	[highprice5y] [float] NULL,
	[lowprice5y] [float] NULL,
	[highprice10y] [float] NULL,
	[lowprice10y] [float] NULL,
	[highpriceytd] [float] NULL,
	[lowpriceytd] [float] NULL,
	[athdate] [date] NULL,
	[atldate] [date] NULL,
	[highprice1ydate] [date] NULL,
	[lowprice1ydate] [date] NULL,
	[highpriceytddate] [date] NULL,
	[lowpriceytddate] [date] NULL,
	[athyield] [float] NULL,
	[atlyield] [float] NULL,
	[highyield1y] [float] NULL,
	[lowyield1y] [float] NULL,
	[highyieldytd] [float] NULL,
	[lowyieldytd] [float] NULL,
	[athyielddate] [date] NULL,
	[atlyielddate] [date] NULL,
	[highyield1ydate] [date] NULL,
	[lowyield1ydate] [date] NULL,
	[highyieldytddate] [date] NULL,
	[lowyieldytddate] [date] NULL,
	[diffbid1d] [float] NULL,
	[diffbid1w] [float] NULL,
	[diff1d] [float] NULL,
	[diffex1d] [float] NULL,
	[diff1w] [float] NULL,
	[diff2w] [float] NULL,
	[diff1m] [float] NULL,
	[diff3m] [float] NULL,
	[diff6m] [float] NULL,
	[diff9m] [float] NULL,
	[diff1y] [float] NULL,
	[diff2y] [float] NULL,
	[diff3y] [float] NULL,
	[diff5y] [float] NULL,
	[diff10y] [float] NULL,
	[diffld] [float] NULL,
	[diffwtd] [float] NULL,
	[diffmtd] [float] NULL,
	[diffqtd] [float] NULL,
	[diffytd] [float] NULL,
	[diffpytd] [float] NULL,
	[diffbid1dprc] [float] NULL,
	[diffbid1wprc] [float] NULL,
	[diff1dprc] [float] NULL,
	[diffex1dprc] [float] NULL,
	[diff1wprc] [float] NULL,
	[diff2wprc] [float] NULL,
	[diff1mprc] [float] NULL,
	[diff3mprc] [float] NULL,
	[diff6mprc] [float] NULL,
	[diff9mprc] [float] NULL,
	[diff1yprc] [float] NULL,
	[diff2yprc] [float] NULL,
	[diff3yprc] [float] NULL,
	[diff5yprc] [float] NULL,
	[diff10yprc] [float] NULL,
	[diffldprc] [float] NULL,
	[diffwtdprc] [float] NULL,
	[diffmtdprc] [float] NULL,
	[diffqtdprc] [float] NULL,
	[diffytdprc] [float] NULL,
	[diffpytdprc] [float] NULL,
	[diffbidyield1d] [float] NULL,
	[diffbidyield1w] [float] NULL,
	[diffyield1d] [float] NULL,
	[diffyield1w] [float] NULL,
	[diffyield2w] [float] NULL,
	[diffyield1m] [float] NULL,
	[diffyield3m] [float] NULL,
	[diffyield6m] [float] NULL,
	[diffyield9m] [float] NULL,
	[diffyield1y] [float] NULL,
	[diffyield2y] [float] NULL,
	[diffyield3y] [float] NULL,
	[diffyield5y] [float] NULL,
	[diffyield10y] [float] NULL,
	[diffyieldld] [float] NULL,
	[diffyieldwtd] [float] NULL,
	[diffyieldmtd] [float] NULL,
	[diffyieldqtd] [float] NULL,
	[diffyieldytd] [float] NULL,
	[diffyieldpytd] [float] NULL,
	[diffbidyield1dprc] [float] NULL,
	[diffbidyield1wprc] [float] NULL,
	[diffyield1dprc] [float] NULL,
	[diffyield1wprc] [float] NULL,
	[diffyield2wprc] [float] NULL,
	[diffyield1mprc] [float] NULL,
	[diffyield3mprc] [float] NULL,
	[diffyield6mprc] [float] NULL,
	[diffyield9mprc] [float] NULL,
	[diffyield1yprc] [float] NULL,
	[diffyield2yprc] [float] NULL,
	[diffyield3yprc] [float] NULL,
	[diffyield5yprc] [float] NULL,
	[diffyield10yprc] [float] NULL,
	[diffyieldldprc] [float] NULL,
	[diffyieldwtdprc] [float] NULL,
	[diffyieldmtdprc] [float] NULL,
	[diffyieldqtdprc] [float] NULL,
	[diffyieldytdprc] [float] NULL,
	[diffyieldpytdprc] [float] NULL,
	[annualizedreturn1y] [float] NULL,
	[annualizedreturn2y] [float] NULL,
	[annualizedreturn3y] [float] NULL,
	[annualizedreturn4y] [float] NULL,
	[annualizedreturn5y] [float] NULL,
	[annualizedreturn10y] [float] NULL,
	[standarddeviation3y] [float] NULL,
	[sharperatio3y] [float] NULL,
	[normanamount] [float] NULL,
	[date] [date] NULL,
	[time] [time](0) NULL,
	CONSTRAINT [performance_pkey] PRIMARY KEY CLUSTERED ([insref] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[keyratios] (
	[insref] [numeric](20, 0) NOT NULL,
	[dividendyield] [float] NULL,
	[per] [float] NULL,
	[per_ttm] [float] NULL,
	[per_last] [float] NULL,
	[psr] [float] NULL,
	[psr_ttm] [float] NULL,
	[psr_last] [float] NULL,
	[pbr] [float] NULL,
	[pbr_last] [float] NULL,
	[pcf] [float] NULL,
	[pcf_ttm] [float] NULL,
	[pcf_last] [float] NULL,
	[dps] [float] NULL,
	[eps] [float] NULL,
	[eps_ttm] [float] NULL,
	[eps_last] [float] NULL,
	[sps] [float] NULL,
	[sps_ttm] [float] NULL,
	[sps_last] [float] NULL,
	[bvps] [float] NULL,
	[bvps_last] [float] NULL,
	[cfps] [float] NULL,
	[cfps_ttm] [float] NULL,
	[cfps_last] [float] NULL,
	[roe] [float] NULL,
	[roe_ttm] [float] NULL,
	[roe_last] [float] NULL,
	[roa] [float] NULL,
	[roa_ttm] [float] NULL,
	[roa_last] [float] NULL,
	[gm] [float] NULL,
	[gm_ttm] [float] NULL,
	[gm_last] [float] NULL,
	[om] [float] NULL,
	[om_ttm] [float] NULL,
	[om_last] [float] NULL,
	[pm] [float] NULL,
	[pm_ttm] [float] NULL,
	[pm_last] [float] NULL,
	[date] [date] NULL,
	[time] [time](0) NULL,
	CONSTRAINT [keyratios_pkey] PRIMARY KEY CLUSTERED ([insref] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[greeks] (
	[insref] [numeric](20, 0) NOT NULL,
	[iv] [float] NULL,
	[ivbid] [float] NULL,
	[ivask] [float] NULL,
	[delta] [float] NULL,
	[gamma] [float] NULL,
	[rho] [float] NULL,
	[theta] [float] NULL,
	[vega] [float] NULL,
	[date] [date] NULL,
	[time] [time](0) NULL,
	CONSTRAINT [greeks_pkey] PRIMARY KEY CLUSTERED ([insref] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[noii] (
	[insref] [numeric](20, 0) NOT NULL,
	[bidprice] [float] NULL,
	[askprice] [float] NULL,
	[bidquantity] [float] NULL,
	[askquantity] [float] NULL,
	[lastprice] [float] NULL,
	[quantity] [float] NULL,
	[imbalance] [float] NULL,
	[direction] [smallint] NULL,
	[type] [smallint] NULL,
	[date] [date] NULL,
	[time] [time](0) NULL,
	CONSTRAINT [noii_pkey] PRIMARY KEY CLUSTERED ([insref] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[constituents] (
	[insref] [numeric](20, 0) NOT NULL,
	[constituent] [numeric](20, 0) NOT NULL,
	[weight] [float] NULL,
	[isin] [varchar](12) NULL,
	[currency] [varchar](4) NULL,
	CONSTRAINT [constituents_pkey] PRIMARY KEY CLUSTERED ([insref] ASC, [constituent] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO
CREATE INDEX [IX_CONSTITUENTS] ON [constituents] ([constituent]);
GO

CREATE TABLE [dbo].[ticksize] (
	[insref] [numeric](20, 0) NOT NULL,
	[threshold] [float] NOT NULL,
	[increment] [float] NOT NULL,
	CONSTRAINT [ticksize_pkey] PRIMARY KEY CLUSTERED ([insref] ASC, [threshold] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[l10n] (
	[insref] [numeric](20, 0) NOT NULL,
	[language] [char](2) NOT NULL,
	[name] [nvarchar](255) NULL,
	[shortdescription] [nvarchar](512) NULL,
	[description] [nvarchar](max) NULL,
	[importantevents] [nvarchar](max) NULL,
	[prospectus] [nvarchar](max) NULL,
	[kiid] [nvarchar](max) NULL,
	[priipkid] [nvarchar](max) NULL,
	[sustaindescription] [nvarchar](max) NULL,
	[sustaininvesting1] [nvarchar](max) NULL,
	[sustaininvesting2] [nvarchar](max) NULL,
	[sustaindivesting1] [nvarchar](max) NULL,
	[sustaindivesting2] [nvarchar](max) NULL,
	[sustaininfluence] [nvarchar](max) NULL,
	[sustainmonitoring1] [nvarchar](max) NULL,
	[sustainmonitoring2] [nvarchar](max) NULL,
	[sustaincomment] [nvarchar](max) NULL,
	[Comprehension_Alert_Portfolio] [char](1) NULL,
	[Intended_target_market_retail_investor_Portfolio] [nvarchar](750) NULL,
	[Investment_objective_Portfolio] [nvarchar](2500) NULL,
	[Risk_narrative_Portfolio] [nvarchar](300) NULL,
	[Other_materially_relevant_risk_narrative_Portfolio] [nvarchar](200) NULL,
	[Type_of_underlying_Investment_Option] [nvarchar](300) NULL,
	[Capital_Guarantee] [char](1) NULL,
	[Capital_Guarantee_Level] [float] NULL,
	[Capital_Guarantee_Limitations] [nvarchar](300) NULL,
	[Capital_Guarantee_Early_Exit_Conditions] [date] NULL,
	[Capital_guarantee_Portfolio] [nvarchar](2500) NULL,
	[Possible_maximum_loss_Portfolio] [float] NULL,
	[Portfolio_Performance_Fees_Narrative] [nvarchar](300) NULL,
	[Portolio_Carried_Interest_Narrative] [nvarchar](300) NULL,
	[Other_comment] [nvarchar](max) NULL,
	[date] [date] NULL,
	CONSTRAINT [l10n_pkey] PRIMARY KEY CLUSTERED ([insref] ASC, [language] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[newscoding] (
	[newsref] [numeric](20, 0) NOT NULL,
	[newsid] [char](36) NOT NULL,
	[isin] [char](12) NOT NULL,
	[company] [numeric](20, 0) NOT NULL,
	[date] [date] NULL,
	CONSTRAINT [PK_newscoding] PRIMARY KEY CLUSTERED ([newsid] ASC, [newsref] ASC, [isin] ASC, [company] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO
CREATE INDEX [IDX_NEWSCODING] ON [newscoding] ([isin],[newsid]);
CREATE INDEX [IDX_NEWSCODING2] ON [newscoding] ([company],[newsid]);
GO

CREATE TABLE [dbo].[newstags] (
	[newsref] [numeric](20, 0) NOT NULL,
	[newsid] [char](36) NOT NULL,
	[tag] [char](30) NOT NULL,
	[date] [date] NOT NULL,
	CONSTRAINT [PK_newstags] PRIMARY KEY CLUSTERED ([newsid] ASC, [newsref] ASC, [tag] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO
CREATE INDEX [IDX_NEWSTAGS] ON [newstags] ([tag],[newsid]);
GO

CREATE TABLE [dbo].[brokers] (
	[insref] [numeric](20, 0) NOT NULL,
	[symbol] [nvarchar](255) NOT NULL,
	[name] [nvarchar](255) NOT NULL,
	CONSTRAINT [brokers_pkey] PRIMARY KEY CLUSTERED ([insref] ASC, [symbol] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[ci] (
	[insref] [numeric](20, 0) NOT NULL,
	[type] [smallint] NOT NULL,
	[subtype] [smallint] NULL,
	[sequence] [int] NOT NULL,
	[s1] [nvarchar](255) NULL,
	[s2] [nvarchar](255) NULL,
	[s3] [nvarchar](255) NULL,
	[s4] [nvarchar](255) NULL,
	[s5] [nvarchar](255) NULL,
	[s6] [nvarchar](255) NULL,
	[s7] [nvarchar](255) NULL,
	[s8] [nvarchar](255) NULL,
	[s9] [nvarchar](255) NULL,
	[s10] [nvarchar](255) NULL,
	[s11] [nvarchar](255) NULL,
	[s12] [nvarchar](255) NULL,
	[s13] [nvarchar](255) NULL,
	[s14] [nvarchar](255) NULL,
	[s15] [nvarchar](255) NULL,
	[n1] [float] NULL,
	[n2] [float] NULL,
	[n3] [float] NULL,
	[n4] [float] NULL,
	[n5] [float] NULL,
	[n6] [float] NULL,
	[n7] [float] NULL,
	[n8] [float] NULL,
	[n9] [float] NULL,
	[n10] [float] NULL,
	[i1] [int] NULL,
	[i2] [int] NULL,
	[i3] [int] NULL,
	[i4] [int] NULL,
	[i5] [int] NULL,
	[i6] [int] NULL,
	[i7] [int] NULL,
	[i8] [int] NULL,
	[i9] [int] NULL,
	[i10] [int] NULL,
	[insref1] [numeric](20, 0) NULL,
	[insref2] [numeric](20, 0) NULL,
	[insref3] [numeric](20, 0) NULL,
	[insref4] [numeric](20, 0) NULL,
	[insref5] [numeric](20, 0) NULL,
	[d1] [date] NULL,
	[d2] [date] NULL,
	[d3] [date] NULL,
	CONSTRAINT [ci_pkey] PRIMARY KEY CLUSTERED ( [insref] ASC, [type] ASC, [sequence] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[cihistory] (
	[insref] [numeric](20, 0) NOT NULL,
	[type] [smallint] NOT NULL,
	[subtype] [smallint] NULL,
	[sequence] [int] NOT NULL,
	[date] [date] NOT NULL,
	[s1] [nvarchar](255) NULL,
	[s2] [nvarchar](255) NULL,
	[s3] [nvarchar](255) NULL,
	[s4] [nvarchar](255) NULL,
	[s5] [nvarchar](255) NULL,
	[s6] [nvarchar](255) NULL,
	[s7] [nvarchar](255) NULL,
	[s8] [nvarchar](255) NULL,
	[s9] [nvarchar](255) NULL,
	[s10] [nvarchar](255) NULL,
	[s11] [nvarchar](255) NULL,
	[s12] [nvarchar](255) NULL,
	[s13] [nvarchar](255) NULL,
	[s14] [nvarchar](255) NULL,
	[s15] [nvarchar](255) NULL,
	[n1] [float] NULL,
	[n2] [float] NULL,
	[n3] [float] NULL,
	[n4] [float] NULL,
	[n5] [float] NULL,
	[n6] [float] NULL,
	[n7] [float] NULL,
	[n8] [float] NULL,
	[n9] [float] NULL,
	[n10] [float] NULL,
	[i1] [int] NULL,
	[i2] [int] NULL,
	[i3] [int] NULL,
	[i4] [int] NULL,
	[i5] [int] NULL,
	[i6] [int] NULL,
	[i7] [int] NULL,
	[i8] [int] NULL,
	[i9] [int] NULL,
	[i10] [int] NULL,
	[insref1] [numeric](20, 0) NULL,
	[insref2] [numeric](20, 0) NULL,
	[insref3] [numeric](20, 0) NULL,
	[insref4] [numeric](20, 0) NULL,
	[insref5] [numeric](20, 0) NULL,
	[d1] [date] NULL,
	[d2] [date] NULL,
	[d3] [date] NULL,
	CONSTRAINT [cihistory_pkey] PRIMARY KEY CLUSTERED ([insref] ASC, [type] ASC, [sequence] ASC, [date] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[priip] (
	[insref] [numeric](20, 0) NOT NULL,
	[Portfolio_Issuer_Name] [nvarchar](255) NULL,
	[Portfolio_Guarantor_Name] [nvarchar](255) NULL,
	[Portfolio_Identifying_Data] [varchar](255) NULL,
	[Type_Of_Identification_Code_For_The_Fund_Share_Or_Portfolio] [smallint] NULL,
	[Portfolio_Name] [nvarchar](255) NULL,
	[Share_Class_Currency] [char](3) NULL,
	[Reference_Date] [date] NULL,
	[Portfolio_PRIIPS_Category] [smallint] NULL,
	[Fund_CIC_code] [char](4) NULL,
	[EOS_portfolio] [char](1) NULL,
	[Valuation_Frequency] [smallint] NULL,
	[Portfolio_VEV_Reference] [float] NULL,
	[IS_Flexible] [char](1) NULL,
	[Flex_VEV_Historical] [float] NULL,
	[Flex_VEV_Ref_Asset_Allocation] [float] NULL,
	[IS_Risk_Limit_Relevant] [char](1) NULL,
	[Flex_VEV_Risk_Limit] [float] NULL,
	[Existing_Credit_Risk] [char](1) NULL,
	[SRI] [smallint] NULL,
	[MRM] [smallint] NULL,
	[CRM] [smallint] NULL,
	[Recommended_Holding_Period] [float] NULL,
	[Maturity_Date] [date] NULL,
	[Liquidity_Risk] [char](1) NULL,
	[Portfolio_return_unfavorable_scenario_1_year] [float] NULL,
	[Portfolio_return_unfavorable_scenario_half_RHP] [float] NULL,
	[Portfolio_return_unfavorable_scenario_RHP] [float] NULL,
	[Portfolio_return_moderate_scenario_1_year] [float] NULL,
	[Portfolio_return_moderate_scenario_half_RHP] [float] NULL,
	[Portfolio_return_moderate_scenario_RHP] [float] NULL,
	[Portfolio_return_favorable_scenario_1_year] [float] NULL,
	[Portfolio_return_favorable_scenario_half_RHP] [float] NULL,
	[Portfolio_return] [float] NULL,
	[Portfolio_return_stress_scenario_1_year] [float] NULL,
	[Portfolio_return_stress_scenario_half_RHP] [float] NULL,
	[Portfolio_return_stress_scenario_RHP] [float] NULL,
	[Portfolio_number_of_observed_return_M0] [float] NULL,
	[Portfolio_mean_observed_returns_M1] [float] NULL,
	[Portfolio_observed_Sigma] [float] NULL,
	[Portfolio_observed_Skewness] [float] NULL,
	[Portfolio_observed_Excess_Kurtosis] [float] NULL,
	[Portfolio_observed_Stressed_Volatility] [float] NULL,
	[One_off_cost_Portfolio_entry_cost] [float] NULL,
	[One_off_cost_Portfolio_entry_cost_Acquired] [float] NULL,
	[One_off_costs_Portfolio_exit_cost_at_RHP] [float] NULL,
	[One_off_costs_Portfolio_exit_cost_at_1_year] [float] NULL,
	[One_off_costs_Portfolio_exit_cost_at_half_RHP] [float] NULL,
	[One_off_costs_Portfolio_sliding_exit_cost_Indicator] [char](1) NULL,
	[Ongoing_costs_Portfolio_other_costs] [float] NULL,
	[Ongoing_costs_Portfolio_management_costs] [float] NULL,
	[Ongoing_costs_Portfolio_transaction_costs] [float] NULL,
	[Existing_performance_fees] [char](1) NULL,
	[Incidental_costs_Portfolio_performance_fees] [float] NULL,
	[Existing_carried_interest_fees] [char](1) NULL,
	[Incidental_costs_Portfolio_carried_interest] [float] NULL,
	[Reference_Language] [char](2) NULL,
	[Comprehension_Alert_Portfolio] [char](1) NULL,
	[Intended_target_market_retail_investor_Portfolio] [nvarchar](750) NULL,
	[Investment_objective_Portfolio] [nvarchar](2500) NULL,
	[Risk_narrative_Portfolio] [nvarchar](300) NULL,
	[Other_materially_relevant_risk_narrative_Portfolio] [nvarchar](200) NULL,
	[Type_of_underlying_Investment_Option] [nvarchar](300) NULL,
	[Capital_Guarantee] [char](1) NULL,
	[Capital_Guarantee_Level] [float] NULL,
	[Capital_Guarantee_Limitations] [nvarchar](300) NULL,
	[Capital_Guarantee_Early_Exit_Conditions] [date] NULL,
	[Capital_guarantee_Portfolio] [nvarchar](2500) NULL,
	[Possible_maximum_loss_Portfolio] [float] NULL,
	[Portfolio_Performance_Fees_Narrative] [nvarchar](300) NULL,
	[Portolio_Carried_Interest_Narrative] [nvarchar](300) NULL,
	[Other_comment] [nvarchar](max) NULL,
	[PRIIP_data_delivery] [char](1) NULL,
	[UCITS_data_delivery] [char](1) NULL,
	[Portfolio_UCITS_SRRI] [smallint] NULL,
	[Portfolio_UCITS_Vol] [float] NULL,
	[Ongoing_costs_Portfolio_other_costs_UCITS] [float] NULL,
	[Ongoing_costs_Portfolio_transaction_costs_UCITS] [float] NULL,
	[Transactions_costs_methodology] [nvarchar](255) NULL,
	[Incidental_costs_Portfolio_performance_fees_UCITS] [float] NULL,
	[Incidental_costs_Portfolio_carried_interest_UCITS] [float] NULL,
	[UCITS_KID_Web_Address] [nvarchar](max) NULL,
	[Bonds_Weight] [float] NULL,
	[Annualized_Return_Volatility] [float] NULL,
	[Duration_Bonds] [float] NULL,
	[Existing_Capital_Preservation] [char](1) NULL,
	[Capital_Preservation_Level] [float] NULL,
	[Time_Interval_Maximum_Loss] [varchar](10) NULL,
	[Uses_PI] [char](1) NULL,
	[Multiplier_PI] [float] NULL,
	[Total_cost_1_year] [float] NULL,
	[RIY_1_year] [float] NULL,
	[Total_cost_half_RHP] [float] NULL,
	[RIY_half_RHP] [float] NULL,
	[Total_cost_RHP] [float] NULL,
	[RIY_RHP] [float] NULL,
	[One_off_costs_Portfolio_entry_cost_RIY] [float] NULL,
	[One_off_costs_Portfolio_exit_cost_RIY] [float] NULL,
	[Ongoing_costs_Portfolio_transaction_costs_RIY] [float] NULL,
	[Ongoing_costs_Other_ongoing_costs_RIY] [float] NULL,
	[Incidental_costs_Portfolio_performance_fees_RIY] [float] NULL,
	[Incidental_costs_Portfolio_carried_interests_RIY] [float] NULL,
	CONSTRAINT [priip_pk] PRIMARY KEY CLUSTERED ([insref] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[mifid] (
	[insref] [numeric](20, 0) NOT NULL,
	[fisn] [varchar](35) NULL,
	[lei] [char](20) NULL,
	[EMT_Version] [char](4) NULL,
	[EMT_Producer_Name] [varchar](255),
	[EMT_Producer_LEI] [char](20),
	[EMT_Producer_Email] [varchar](255),
	[File_Generation_Date_and_Time] [datetime] NULL,
	[EMT_Data_Reporting_Target_Market] [char](1) NULL,
	[EMT_Data_Reporting_ExAnte] [char](1) NULL,
	[EMT_Data_Reporting_ExPost] [char](1) NULL,
	[Financial_Instrument_Identifying_Data] [varchar](255) NULL,
	[Type_Of_Identification_Code_For_The_Financial_Instrument] [smallint] NULL,
	[Financial_Instrument_Name] [nvarchar](255) NULL,
	[Financial_Instrument_Currency] [char](3) NULL,
	[Financial_Instrument_Performance_Fee] [char](1) NULL,
	[Financial_Instrument_Distribution_Of_Cash] [char](1) NULL,
	[Reporting_Date] [date] NULL,
	[Financial_Instrument_Legal_Structure] [varchar](2) NULL,
	[Financial_Instrument_Issuer_Name] [nvarchar](255) NULL,
	[Financial_Instrument_Manufacturer_LEI] [varchar](20) NULL,
	[Financial_Instrument_Manufacturer_Email] [nvarchar](255) NULL,
	[Financial_Instrument_Manufacturer_Product_Governance_Process] [char](1) NULL,
	[Financial_Instrument_Guarantor_Name] [nvarchar](255) NULL,
	[Financial_Instrument_Type_Notional_or_Item_Based] [char](1) NULL,
	[Product_Category_or_Nature] [nvarchar](255) NULL,
	[Structured_Securities_Product_Category_or_Nature] [int] NULL,
	[Structured_Securities_Quotation] [char](1) NULL,
	[Leveraged_Financial_Instrument_or_Contingent_LiabilityInstrumen] [char](1) NULL,
	[Fund_Share_Class_Without_Retrocession] [char](1) NULL,
	[Ex_Post_Cost_Calculation_Basis_Italy] [char](1) NULL,
	[Target_Market_Reference_Date] [date] NULL,
	[Investor_Type_Retail] [char](1) NULL,
	[Investor_Type_Professional] [char](1) NULL,
	[Investor_Type_Eligible_Counterparty] [char](1) NULL,
	[Basic_Investor] [char](1) NULL,
	[Informed_Investor] [char](1) NULL,
	[Advanced_Investor] [char](1) NULL,
	[Expert_Investor_Germany] [char](1) NULL,
	[No_Capital_Loss] [char](1) NULL,
	[Limited_Capital_Loss] [char](1) NULL,
	[Limited_Capital_Loss_Level] [float] NULL,
	[No_Capital_Guarantee] [char](1) NULL,
	[Loss_Beyond_Capital] [char](1) NULL,
	[Risk_Tolerance_PRIIPS_Methodology] [smallint] NULL,
	[Risk_Tolerance_UCITS_Metholodology] [smallint] NULL,
	[Risk_Tolerance_Internal_Methodology_For_Non_PRIIPS_and_NonUCITS] [char](1) NULL,
	[Risk_Tolerance_For_Non_PRIIPS_and_Non_UCITS_Spain] [smallint] NULL,
	[Not_For_Investors_With_The_Lowest_Risk_Tolerance_Germany] [char](1) NULL,
	[Return_Profile_Preservation] [char](1) NULL,
	[Return_Profile_Growth] [char](1) NULL,
	[Return_Profile_Income] [char](1) NULL,
	[Return_Profile_Hedging] [char](1) NULL,
	[Option_or_Leveraged_Return_Profile] [char](1) NULL,
	[Return_Profile_Other] [char](1) NULL,
	[Return_Profile_Pension_Scheme_Germany] [char](1) NULL,
	[Time_Horizon1] [char](1) NULL,
	[Time_Horizon2] [float] NULL,
	[Maturity_Date] [date] NULL,
	[May_Be_Terminated_Early] [char](1) NULL,
	[Intended_Compatible_With_Clients_Having_ESG_Preferences] [char](1) NULL,
	[Specific_Investment_Need] [char](1) NULL,
	[Other_Specific_Investment_Need] [char](1) NULL,
	[Execution_Only] [char](1) NULL,
	[Execution_With_Appropriateness_Test_Or_Non_Advised_Services] [char](1) NULL,
	[Investment_Advice] [char](1) NULL,
	[Portfolio_Management] [char](1) NULL,
	[OO_Financial_Instrument_entry_cost] [float] NULL,
	[Net_OO_cost_Financial_Instrument_entry_cost_non_acquired] [float] NULL,
	[OO_Financial_Instrument_maximum_entry_cost_fixed_amount_Italy] [float] NULL,
	[OO_Financial_Instrument_maximum_entry_cost_acquired] [float] NULL,
	[OO_Financial_Instrument_maximum_exit_cost] [float] NULL,
	[OO_Financial_Instrument_maximum_exit_cost_fixed_amount_Italy] [float] NULL,
	[OO_Financial_Instrument_maximum_exit_cost_acquired] [float] NULL,
	[OO_Financial_Instrument_Typical_exit_cost] [float] NULL,
	[OO_Financial_Instrument_exit_cost_structrd_securities_prior_RHP] [float] NULL,
	[Financial_Instrument_Ongoing_costs] [float] NULL,
	[Financial_Instrument_Gearing_costs_ex_ante_UK] [float] NULL,
	[Financial_Instrument_Management_fee] [float] NULL,
	[Financial_Instrument_Distribution_fee] [float] NULL,
	[Financial_Instrument_Transaction_costs_ex_ante] [float] NULL,
	[Financial_Instrument_Incidental_costs_ex_ante] [float] NULL,
	[Structured_Securities_Reference_Price_ex_ante] [float] NULL,
	[Structured_Securities_Notional_Reference_Amount_ex_ante] [float] NULL,
	[Ex_Ante_Costs_Reference_Date] [date] NULL,
	[OO_Structured_Securities_entry_cost_ex_post] [float] NULL,
	[Net_OO_cost_Structured_Securities_entry_cost_ex_post] [float] NULL,
	[OO_Structured_Securities_exit_cost_ex_post] [float] NULL,
	[OO_cost_Financial_Instrument_entry_cost_acquired] [float] NULL,
	[Financial_Instrument_Ongoing_costs_ex_post] [float] NULL,
	[Structured_Securities_Ongoing_costs_ex_post_accumulated] [float] NULL,
	[Financial_Instrument_Borrowing_Costs_Ex_Post_UK] [float] NULL,
	[Financial_Instrument_Management_fee_ex_post] [float] NULL,
	[Financial_Instrument_Distribution_fee_ex_post] [float] NULL,
	[Financial_Instrument_Transaction_costs_ex_post] [float] NULL,
	[Financial_Instrument_Incidental_costs_ex_post] [float] NULL,
	[Beginning_Of_Calculation_Period] [date] NULL,
	[End_Of_Calculation_Period] [date] NULL,
	[Structured_Securities_Reference_Price_ex_post] [float] NULL,
	[Structured_Securities_Notional_Reference_Amount] [float] NULL,
	CONSTRAINT [mifid_pk] PRIMARY KEY CLUSTERED ([insref] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[mifidhistory] (
	[insref] [numeric](20, 0) NOT NULL,
	[Reporting_Date] [date] NOT NULL,
	[OO_Structured_Securities_entry_cost_ex_post] [float] NULL,
	[Net_OO_cost_Structured_Securities_entry_cost_ex_post] [float] NULL,
	[OO_Structured_Securities_exit_cost_ex_post] [float] NULL,
	[OO_cost_Financial_Instrument_entry_cost_acquired] [float] NULL,
	[Financial_Instrument_Ongoing_costs_ex_post] [float] NULL,
	[Structured_Securities_Ongoing_costs_ex_post_accumulated] [float] NULL,
	[Financial_Instrument_Borrowing_Costs_Ex_Post_UK] [float] NULL,
	[Financial_Instrument_Management_fee_ex_post] [float] NULL,
	[Financial_Instrument_Distribution_fee_ex_post] [float] NULL,
	[Financial_Instrument_Transaction_costs_ex_post] [float] NULL,
	[Financial_Instrument_Incidental_costs_ex_post] [float] NULL,
	[Beginning_Of_Calculation_Period] [date] NULL,
	[End_Of_Calculation_Period] [date] NULL,
	[Structured_Securities_Reference_Price_ex_post] [float] NULL,
	[Structured_Securities_Notional_Reference_Amount] [float] NULL,
	CONSTRAINT [mifidhistory_pk] PRIMARY KEY CLUSTERED ([insref] ASC, [Reporting_Date] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[underlyings] (
	[insref] [numeric](20, 0) NOT NULL,
	[isin] [varchar](12) NOT NULL,
	[currency] [varchar](4) NULL,
	CONSTRAINT [underlyings_pkey] PRIMARY KEY CLUSTERED ([insref] ASC, [isin] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO
CREATE INDEX [IX_UNDERLYINGS] ON [underlyings] ([isin]);
GO

CREATE TABLE [dbo].[sectors] (
	[insref] [numeric](20, 0) NOT NULL,
	[sector] [numeric](20, 0) NOT NULL,
	[marketplace] [numeric](20, 0) NOT NULL,
	CONSTRAINT [sectors_pkey] PRIMARY KEY CLUSTERED ([insref] ASC, [sector] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO
CREATE INDEX [IX_SECTORS] ON [sectors] ([sector], [marketplace]);
GO

CREATE TABLE [dbo].[combolegs] (
	[insref] [numeric](20, 0) NOT NULL,
	[leg] [numeric](20, 0) NOT NULL,
	[ratio] [int] NULL,
	[side] [smallint] NULL,
	CONSTRAINT [combolegs_pkey] PRIMARY KEY CLUSTERED ([insref] ASC, [leg] ASC) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

